function [ aberration ] = m_gen_PSF_aberration( varargin )
% This function calculates the wavefront aberration function using Zernike
% polynomials on unit circle and the Lanni-Gibson aberration caused by the
% stratified medium. Only the defocus and the tilting Zernike terms
% calculated for each molecule, the other calculated only once. Calling
% this function for the Lanni-Gibson aberration calculation also should
% happen only once.

% PARENT: m_gen_PSF_aperture_func
% CHILD: -
aberration_type=varargin{1};    % aberration type to calculate
params=varargin{2};             % parameters

% read the right parameters for the aberration calculation
switch aberration_type
    case 'Zernike_pos'  % if the function was called for Zernike aberration calculation
        x_tube_norm=varargin{3};  % normalized sample points on the tube lens' aperture
        y_tube_norm=varargin{4};  % normalized sample points on the tube lens' aperture
        ro_tube_2=varargin{5};   % square of the radius of the normalized sample points on the objective's aperture
    case 'Zernike_const'
        x_tube_norm=varargin{3};  % normalized sample points on the objective's aperture
        y_tube_norm=varargin{4};  % normalized sample points on the objective's aperture
        ro_tube_2=varargin{5};   % square of the radius of the normalized sample points on the objective's aperture
    case 'Lanni_Gibson'    % if the function was called for Lanni-Gibson aberration calculation
        %theta_s=varargin{3};    % ray angles in the sample
        %theta_i=varargin{4};    % ray angles in the immersion medium
        ro_tube_2=varargin{3};   % square of the radius of the normalized sample points on the objective's aperture
        circ=varargin{4};
        delta_d_i=varargin{5};
        NA_LG=varargin{6};
    case 'astigmatism'
        y_tube_norm=varargin{3};  % normalized sample points on the objective's aperture
    case 'Fourier_mask'
        % no yet implemented
    otherwise
        error('Something is wrong with the aberration type.')
end

% calculating the Zernike polinomials and coefficients
switch aberration_type
    case 'Zernike_pos'
        
        % "exp(-i*k*x)" convention is used
        
        z_pos=params.mol.pos(3)*1E-9;    % "z" position of the molecule in m 

        % the position of the molecule is shifted to the "central" pixels
        % in order to be able to use the same spatial frequencies in the
        % discrete Fourier-transformt
        x_pos_rem=params.numcalc.psf_pos_rem_x;  % "x" position of the molecule after the shift in m
        y_pos_rem=params.numcalc.psf_pos_rem_y;  % "y" position of the molecule after the shift in m
        
        aperture_rad_tube=params.numcalc.aperture_rad_tube;     % modified radius of the tube lens, equals "NA"
        z_focal_x=params.numcalc.z_focal_x;
        
        n_s=params.aq.n_sample;      % refractive index of the immersion medium
        
        % Zerinke polynomials and terms
        Z_1=2*x_tube_norm;        % phasefront tilt in x dimension
        z_1=-0.5*x_pos_rem*aperture_rad_tube;
        
        Z_2=2*y_tube_norm;        % phasefront tilt in y dimension
        z_2=-0.5*y_pos_rem*aperture_rad_tube;
        
        Z_3=2*ro_tube_2-1;      % defocus

        z_3=-0.5*(0.5*aperture_rad_tube^2/n_s*(z_pos+z_focal_x));
        % "z_focal_x" is focused to the detector plane
        % "z_pos-z_focal_x" is the "z" position of the molecule in the coordinate system shifted to the focal point in the "x-z" plane
        
        aberration=z_1*Z_1+z_2*Z_2+z_3*Z_3;

    case 'Zernike_const'
        
        % calculating the constant terms of the Zernike aberration
        Z_4=(x_tube_norm.^2-y_tube_norm.^2);      % primary astigmatism, x?
        z_4=0;

        Z_5=2*x_tube_norm.*y_tube_norm;           % primary astigmatism, y?
        z_5=0;

        Z_6=(3*ro_tube_2-2).*x_tube_norm;   % primary coma, x?
        z_6=0;

        Z_7=(3*ro_tube_2-2).*y_tube_norm;   % primary coma, y?
        z_7=0;

        Z_8=6*(ro_tube_2.^2-ro_tube_2)+1;   % primary spherical aberration
        z_8=0;

        aberration=z_4*Z_4+z_5*Z_5+z_6*Z_6+z_7*Z_7+z_8*Z_8;    % the Zernike aberration
        
    case 'Lanni_Gibson'

        n_i=params.aq.n_immersion;      % refractive index of the immersion medium
        n_s=params.aq.n_sample;        % refractive index the sample, let it be the refractive index of water at 650nm

        d_s=params.psf.sample_depth*1E-6;         % depth of focal point in the sample in m
        
        Lanni_Gibson_ab=d_s*sqrt(n_s^2-NA_LG^2*ro_tube_2.*circ)+delta_d_i*sqrt(n_i^2-NA_LG^2*ro_tube_2.*circ);  % Gibson and Lanni phase aberration term
        
        aberration=Lanni_Gibson_ab;
        
        % checking the Lanni-Gibson aberration
        %figure()
        %mesh(abs(Lanni_Gibson_ab))
        %figure()
        %mesh(real(Lanni_Gibson_ab))
        %figure()
        %mesh(imag(Lanni_Gibson_ab))
        
    case 'astigmatism'
        
        % "exp(-i*k*x)" convention is used
        
        aperture_rad_tube=params.numcalc.aperture_rad_tube;     % modified radius of the tube lens, equals "NA"
        z_focal_x=params.numcalc.z_focal_x;     % "z" coordinate of the focal point in the "x-z" plane (tube lens' focal point) in object space coordinates
        z_focal_y=params.numcalc.z_focal_y;     % "z" coordinate of the focal point in the "y-z" plane (joint focal point of the tube and cylindrical lens) in object space coordinates
        
        n_s=params.aq.n_sample;      % refractive index of the immersion medium
        
        aberration=-0.5*y_tube_norm.^2*aperture_rad_tube^2/n_s*(z_focal_y-z_focal_x);    % phase front aberration caused by the cylindrical lens
        % "z_focal_y-z_focal_x" (after multiplied with a positive constant) gives how much closer the "f_y" located to the tube lens than the "f_x" in the object space
        % "z_focal_y-z_focal_x" is "z" the difference between the focal points in the "y-z" and "y-z" place in that order in object space coordinates

    case 'Fourier_mask'
        if false
            % no yet implemented
        else
           aberration=0; 
        end
        
    otherwise
        error('Something is wrong with the aberration type.')
end

end

        
