function [labels] = m_gen_mol_coord_axons( vert_coords, params )
% This function places the contact points on the axons. Contact points are
% on "parallel "circles located on cylindrical surfaces (axons) arranged
% equidistantly. It goes through the axons, places the circles on them
% equidistantly and in parallel, places the contact points on the circles
% randomly, eventualy connects the dye molecules to the contact points.

% PARENT: m_gen_mol_coord.m
% CHILDS: m_gen_linker.m, m_gen_linker_rot.m

% VARIABLES
% radius: radius of cylindrical surfaces
% dens: density of contact points on the circles of the individual axons
% circ_dist: distance between the adjacent circles on the individual axons
% N_cont_circ: number of contact point on one circle on the individual axons
% N_axon: number of cylindrical surfaces
% vect_axon: vector pointing from the starting point to the ending point of the cylindrical surfaces
% length_axon: lengths of cylindrical surfaces
% idx_axon: goes through the cylindrical surfaces
% theta: polar angle of the vector parallel with the actual segment
% fi: azimuth angle of the vector parallel with the actual segment
% R_y: rotation matrix of the actual axon around the "y" axis   
% R_z: rotation matrix of the actual axon around the "z" axis
% idx_circ: index of the actual circle on the actual axon
% cont_vect: vector pointing from the axon's starting point to te actual circle's center
% idx_cont_axon: index contact points on the actual circle
% alpha: random polar angle on the circle of the actual contact point
% contact_point_axon: vector pointing from the axon's axis to the actual contact point
% contact_point: coordinates of the actual contact point
% r2: radius of the actual axon, should be equal to (radius(idx_axon))
% theta2: polar angle of surface normal
% fi2: azimuth angle of surface normal
% R_y2: rotation matrix of the linker around the "y" axis
% R_z2: rotation matrix of the linker around the "z" axis
% tangent_vect: vector tangent to the actual actin ring at the actual contact point
% theta3:  polar angle of the actual ring tangent vector
% fi3:  azimuth angle of the actual ring tangent vector
% beta: the angle that gives how much the "x" unit vector be rotated in order it to be parallel with the ring tangent vector after rotating "z" unit vector to be parallel with the normal vector
% beta_rand: used to randomly invert or not invert the ring tangent vector
% R_0: rotation matrix of the "x" unit vector, should be applied before the rotation matrices used to rotate the "z" to be parallel with the normal vector
% idx_mol_cont: local index of the molecule on the actual contact point
% idx_mol: index of the actual molecule
% link_vec: vector of the actual dye's linker
% labels.init_dye_coords: variable containing dye coordinates in 3xM_mol array
% labels.init_orientation:  variable containing dye orientations (azimuth, polar angles) in 3xM_mol array
% labels.init_cont_coords: variable containing contact point coordinates in 3XM_cont array

radius=params.sample.axon_radius;   % radius of cylindrical surfaces
if radius==0
   error('Axons can not have zero radii.') 
end
dens=params.sample.label_dens*1E-3; % density of contact points on the circles of the individual axons
circ_dist=params.sample.axon_grating;   % distance between the adjacent circles on the individual axons

N_cont_circ=round(2*pi*radius.*dens);   % number of contact point on one circle on the individual axons

N_axon=length(vert_coords);         % number of cylindrical surfaces

vect_axon=zeros(3,N_axon);      % vector pointing from the starting point to the ending point of the cylindrical surfaces
length_axon=zeros(1,N_axon);    % lengths of cylindrical surfaces
% caculating these variables:
for idx_axon=1:N_axon
%for idx_axon=3:N_axon
    vect_axon(:,idx_axon)=vert_coords{idx_axon}(:,2)-vert_coords{idx_axon}(:,1);
    length_axon(1,idx_axon)=norm(vect_axon(:,idx_axon));
end

N_circ_axon=floor(length_axon./circ_dist)+1;    % number of circles on the individual axons

N_cont=sum(N_circ_axon.*N_cont_circ);       % total number of contact points

[labels, N_mol_cont, link_vec_sign]=m_gen_linker(N_cont, params);

idx_cont=0;      % index of the actual contact point
idx_mol=0;      % index of the actual dye molecule
for idx_axon=1:N_axon           % goes through the cylindrical surfaces
%for idx_axon=3:N_axon
    theta=acos(vect_axon(3,idx_axon)/length_axon(idx_axon));   % polar angle of the vector parallel with the actual segment
    fi=atan2(vect_axon(2,idx_axon),vect_axon(1,idx_axon));   % azimuth angle of the vector parallel with the actual segment
    if isnan(fi)    % if the axon's axis is parallel with the "z" axis...
        fi=0;       % let the azimuth angle be zero
    end
    % rotation matrix of the actual axon around the "y" axis
    R_y=[cos(theta),0,sin(theta);...
        0,1,0;...
        -sin(theta),0,cos(theta)];
    % rotation matrix of the actual axon around the "z" axis
    R_z=[cos(fi),-sin(fi),0;...
        sin(fi),cos(fi),0;...
        0,0,1];
    for idx_circ=1:N_circ_axon(idx_axon)    % goes through the circles on the actual axon
        cont_vect=(idx_circ-1)*circ_dist(idx_axon)*vect_axon(:,idx_axon)/norm(vect_axon(:,idx_axon)); % vector pointing from the axon's starting point to te actual circle's center
        for idx_cont_axon=1:N_cont_circ(idx_axon)   % goes through the contact points on the actual circle
            idx_cont=idx_cont+1;
            % placing the contact point on the circle randomly
            alpha=2*pi*rand(1);
            contact_point_axon=R_z*R_y*[cos(alpha);sin(alpha);0]*radius(idx_axon);
            % shifting the contact point to right position
            contact_point=contact_point_axon+cont_vect+vert_coords{idx_axon}(:,1);
            
            % calculating ratation matrices of the surface normal of the actual contact point
            r2=norm(contact_point_axon);    % radius of the actual axon
            theta2=acos(contact_point_axon(3)/r2); % polar angle of surface normal
            fi2=atan2(contact_point_axon(2),contact_point_axon(1)); % azimuth angle of surface normal
            if isnan(fi2)   % if the vector is parallel with the "z" axis...
                fi2=0;      % let the azimuth angle be zero
            end
            % rotation matrix of the linker around the "y" axis
            R_y2=[cos(theta2),0,sin(theta2);...
            0,1,0;...
            -sin(theta2),0,cos(theta2)];
            % rotation matrix of the linker around the "z" axis
            R_z2=[cos(fi2),-sin(fi2),0;...
            sin(fi2),cos(fi2),0;...
            0,0,1];
        
            tangent_vect=cross(contact_point_axon, vect_axon(:,idx_axon));      % vector tangent to the actin ring at the contact point
            
            theta3=acos(tangent_vect(3)/norm(tangent_vect)); % polar angle of the ring tangent vector
            fi3=atan2(tangent_vect(2),tangent_vect(1)); % azimuth angle of the ring tangent vector
            if isnan(fi3)   % if the vector is parallel with the "z" axis...
                fi3=0;      % let the azimuth angle be zero
            end
        
            % calculating the angle that gives how much the "x" unit vector
            % be rotated in order it to be parallel with the ring tangent
            % vector after rotating "z" unit vector to be parallel with the
            % normal vector
            % "beta" was calculated from the condition:
            % ((R_z2*R_y2*[1;0;0])x[tangent or axis vector])*[normal vector]=0
            % the cross product should be enough, scalar product is used only for safety reasons
            beta=atan2((sin(fi2)*sin(theta3)*cos(fi3)-cos(fi2)*sin(theta3)*sin(fi3)),...
                (cos(theta3)*sin(theta2)-sin(fi2)*sin(theta3)*sin(fi3)*cos(theta2)-cos(fi2)*sin(theta3)*cos(fi3)*cos(theta2)));
            % randomly invert or not invert the tangent vector
            beta_rand=randi([0,1],1)*pi;
            % rotation matrix of the "x" unit vector, should be applied
            % before the rotation matrices used to rotate the "z" to be
            % parallel with the normal vector
            R_0=[cos(beta+beta_rand), -sin(beta+beta_rand), 0;...
                sin(beta+beta_rand), cos(beta+beta_rand), 0;...
                0, 0, 1];
        
            % rotating the linkers and connecting them to the contact points
            [labels, idx_mol]=m_gen_linker_rot(params, labels, link_vec_sign, R_0, R_y2, R_z2,  N_mol_cont(idx_cont), idx_mol, contact_point);
            
            labels.init_cont_coords(:,idx_cont)=contact_point;  % variable containing contact point coordinates in 3XN_cont array
        end
    end
end

end

