function [labels] = m_gen_mol_coord_tubules( vert_coords, params)
% This function places the contact points on microtubules. The microtubules
% are consist of cylindrical surfaces around segments. Firs calls a
% function which places the contact point on segmented lines in uniform
% distribution. Then the contact points are shifted perpendicular to the
% segment with the radius value and rotated with a random angle around the
% segment. When two nonparallel segments join, there is an overlapping
% area. Calculets the length of the overlapping are for each segment, the
% so called "affected area". Examines the contact points (using dot and
% cross products and projections) in the affected area whether they really
% are in the overlapping area. If the contact points are in the overlapping
% area, the vector pointing to the contact point from the starting or
% ending point of the segment ("cont_vect_vert") is mirrored.

% PARENT: m_gen_mol_coord.m
% CHILDS: m_gen_mol_coord_segments.m, m_gen_linker.m, m_gen_linker_rot.m

% VARIABLES
% N_cont: total number of contact point
% idx_cont: index of the actual contact point
% idx_mol: index of the actual dye molecule
% idx_curve: index of the actual curve
% idx_seg: index of the actual segment
% idx_mol: index of the dye molecule on the segment
% coords{}(): coordinates of the line's verices
% contact_points_seg: initial contact points of the dye molecules on the segments
% radius: radius of the tubules
% N_mol_seg{idx_curve}(idx_seg): number of dye molecules on the segment
% seg_length{idx_curve}(idx_seg): length of the segment in nm
% par_vect_act: vector parallel with the actual segment
% par_vect_prev: vector parallel with the previous segment
% par_vect_next: vector parallel with the next segment
% delta_prev: angle between the actual and the previous segment
% delta_next: angle between the actual and the next segment
% matching_length_prev: the length of the part of the segment which is affected by connection of the actual and the previous segment
% matching_length_next: the length of the part of the segment which is affected by connection of the actual and the next segment
% bisector_prev: bisector of the "delta_prev" angle
% bisector_next: bisector of the "delta_next" angle
% theta: polar angle of the segment's axis 
% fi: azimuth angle of the segment's axis 
% R_y: rotation matrix around the "y" axis for  placing the contact point on the cylindrical surface
% R_z: rotation matrix around the "z" axis for  placing the contact point on the cylindrical surfac
% beta: the angle that gives how much the "x" unit vector be rotated in order it to be parallel with the axis vector after rotating "z" unit vector to be parallel with the normal vector
% beta_rand: used to randomly invert or not invert the axis vector
% R_0: rotation matrix of the "x" unit vector, should be applied before the rotation matrices used to rotate the "z" to be parallel with the normal vector
% idx_cont_seg: local index of the contact point on the actual segment
% alpha: random variable for uniformly placing the contact point on the cylindrical surface 
% cont_unit_vert_seg: unit vector normal to the surface where the contact point is located
% contact_point: coordinates of the actual contact point
% cont_vect_vert: vector pointing from the "strarting point" of the segment to the actual contact point
% norm_vect: unit vector normal to the two connecting segments's plain
% cont_vect_proj: coordinates of the contact point projected into the two connecting segments's plain
% r2: length of the unit vector normal to the surface where the contact point is located
% theta2: polar angle of the unit vector normal to the surface where the contact point is located
% fi2: azimuth angle of the unit vector normal to the surface where the contact point is located
% R_y2: rotation matrix for the linkers
% R_z2: rotation matrix for the linkers


if strcmp(params.sample.name,'lines')
    radius=params.sample.lines_radius;
elseif strcmp(params.sample.name,'segments')
    radius=params.sample.seg_radius;
else
end

% calculating the initial position of the contact points
[contact_points_seg,N_cont_seg,seg_length ]=m_gen_mol_coord_segments(vert_coords, params, 1.0);

N_cont=sum(cell2mat(cellfun(@(x) sum(x),N_cont_seg,'UniformOutput',false)));    % total number of conteact poits

% getting the linker vectors
[labels, N_mol_cont, link_vec_sign]=m_gen_linker(N_cont, params);

idx_cont=0;
idx_mol=0;
% goes through the different curves 
for idx_curve=1:length(N_cont_seg)
    % goes through the different segments on the curve
    for idx_seg=1:length(N_cont_seg{idx_curve})
        par_vect_act=vert_coords{idx_curve}(:,idx_seg+1)-vert_coords{idx_curve}(:,idx_seg);
        % if it's the first segment of the curve
        if and(idx_seg==1,length(seg_length{idx_curve})>1)
            par_vect_prev=zeros(3,1);     % there is no previous
            par_vect_next=vert_coords{idx_curve}(:,idx_seg+2)-vert_coords{idx_curve}(:,idx_seg+1);
            delta_prev=0;     % there is no previous
            delta_next=acos(dot(-par_vect_act,par_vect_next)/...
                (seg_length{idx_curve}(idx_seg)*seg_length{idx_curve}(idx_seg+1)))/2.;
            % if the next segment is parallel with the actual segmment...
            if delta_next==0
                matching_length_next=-1;    %the cylinders of the segments do not overlap
            % otherwise calcules the length of the overlapping area
            else
                matching_length_next=radius/tan(delta_next);
            end
            matching_length_prev=-1.;   % the cylinders of the actual and the non-existing previous segments do not overlap
            bisector_prev=zeros(3,1);
            bisector_next=-par_vect_act+par_vect_next;
        % if it is the last segment of the curve
        elseif and(idx_seg==length(N_cont_seg{idx_curve}),length(seg_length{idx_curve})>1)
            par_vect_prev=vert_coords{idx_curve}(:,idx_seg)-vert_coords{idx_curve}(:,idx_seg-1);
            par_vect_next=zeros(3,1);
            delta_prev=acos(dot(-par_vect_prev,par_vect_act)/...
                (seg_length{idx_curve}(idx_seg-1)*seg_length{idx_curve}(idx_seg)))/2.;
            % if the next segment is parallel with the actual segmment...
            if delta_prev==0
                matching_length_prev=-1;   % the cylinders of the segments do not overlap
           % otherwise calcules the length of the overlapping area
            else
                matching_length_prev=radius/tan(delta_prev);
            end
            delta_next=0;
            matching_length_next=-1.;   % the cylinders of the actual and the non-existing next segments do not overlap
            bisector_prev=-par_vect_prev+par_vect_act;
            bisector_next=zeros(3,1);
        % if it is neither the first nor the last segment of the curve
        elseif length(seg_length{idx_curve})>2
            par_vect_prev=vert_coords{idx_curve}(:,idx_seg)-vert_coords{idx_curve}(:,idx_seg-1);
            par_vect_next=vert_coords{idx_curve}(:,idx_seg+2)-vert_coords{idx_curve}(:,idx_seg+1);
            delta_prev=acos(dot(-par_vect_prev,par_vect_act)/...
                (seg_length{idx_curve}(idx_seg-1)*seg_length{idx_curve}(idx_seg)))/2.;
            if delta_prev==0
                matching_length_prev=-1;
            else
                matching_length_prev=radius/tan(delta_prev);
            end
            delta_next=acos(dot(-par_vect_act,par_vect_next)/...
                (seg_length{idx_curve}(idx_seg)*seg_length{idx_curve}(idx_seg+1)));
            if delta_next==0
                matching_length_next=-1;
            else
                matching_length_next=radius/tan(delta_next);
            end
            bisector_prev=-par_vect_prev+par_vect_act;
            bisector_next=-par_vect_act+par_vect_next;
        % in every other case...
        else
            matching_length_prev=-1;
            matching_length_next=-1;
        end
        
        theta=acos(par_vect_act(3)/seg_length{idx_curve}(idx_seg));   % polar angle of the vector parallel with the actual segment
        fi=atan2(par_vect_act(2),par_vect_act(1));   % azimuthal angle of the vector parallel with the actual segment
        if isnan(fi)   % if the vedtor is parallel with the "z" axis...
            fi=0;      % let the azimuth agnle be zero
        end
        % rotation mattrix around the "y" axis
        R_y=[cos(theta),0,sin(theta);...
            0,1,0;...
            -sin(theta),0,cos(theta)];
        % rotation matrix around the "z" axis
        R_z=[cos(fi),-sin(fi),0;...
            sin(fi),cos(fi),0;...
            0,0,1];
        % goes through the different dye molecules on the actual segment
        for idx_cont_seg=1:N_cont_seg{idx_curve}(idx_seg)
            idx_cont=idx_cont+1;
            alpha=rand(1)*2*pi;
            cont_unit_vert_seg=R_z*R_y*[cos(alpha);sin(alpha);0];   % unit vector normal to the surface where the contact point is located
            contact_point=contact_points_seg(:,idx_cont)+cont_unit_vert_seg*radius;
            % if the initial contact point is in the affected part
            if norm(contact_points_seg(:,idx_cont)-vert_coords{idx_curve}(:,idx_seg))<matching_length_prev
                cont_vect_vert=contact_point-vert_coords{idx_curve}(:,idx_seg); % vector pointing from the "strarting point" of the segment to the actual contact point
                norm_vect=cross(bisector_prev,par_vect_act);
                norm_vect=norm_vect/norm(norm_vect);
                cont_vect_proj=cont_vect_vert-norm_vect*dot(norm_vect,cont_vect_vert);
                % decide whether the contact point is affected
                if dot(norm_vect,cross(bisector_prev, cont_vect_proj))<0
                    cont_vect_vert=-cont_vect_vert;
                    contact_point=vert_coords{idx_curve}(:,idx_seg)+cont_vect_vert;
                end
            % if the initial contact point is in the other affected part
            elseif norm(contact_points_seg(:,idx_cont)-vert_coords{idx_curve}(:,idx_seg+1))<matching_length_next
                cont_vect_vert=contact_point-vert_coords{idx_curve}(:,idx_seg+1); % vector pointing from the "strarting point" of the segment to the actual contact point
                norm_vect=cross(bisector_next,par_vect_act);
                cont_vect_proj=cont_vect_vert-norm_vect*dot(norm_vect,cont_vect_vert);
                % deciding whether the contact point is affected
                if dot(norm_vect,cross(bisector_next, cont_vect_proj))>0
                    cont_vect_vert=-cont_vect_vert;
                    contact_point=vert_coords{idx_curve}(:,idx_seg+1)+cont_vect_vert;
                end
            end
            % angles for the rotation matrices of the linkers
            r2=norm(cont_unit_vert_seg);    % length of this vector should be unite, but for safety reasons...
            theta2=acos(cont_unit_vert_seg(3)/r2);
            fi2=atan2(cont_unit_vert_seg(2),cont_unit_vert_seg(1));
            if isnan(fi2)   % if the vedtor is parallel with the "z" axis...
                fi2=0;      % let the azimuth agnle be zero
            end

            % rotation matrices, the base of the linker's half spehere
            % needs to be tangential to the microtubule's surface
            R_y2=[cos(theta2),0,sin(theta2);...
            0,1,0;...
            -sin(theta2),0,cos(theta2)];
            R_z2=[cos(fi2),-sin(fi2),0;...
            sin(fi2),cos(fi2),0;...
            0,0,1];
        
            % calculating the angle that gives how much the "x" unit vector
            % be rotated in order it to be parallel with the axis  vector
            % after rotating "z" unit vector to be parallel with the normal
            % vector
            % "beta" was calculated from the condition:
            % ((R_z2*R_y2*[1;0;0])x[tangent or axis vector])*[normal vector]=0
            % the cross product should be enough, scalar product is used only for safety reasons
            beta=atan2((sin(fi2)*sin(theta)*cos(fi)-cos(fi2)*sin(theta)*sin(fi)),...
                (cos(theta)*sin(theta2)-sin(fi2)*sin(theta)*sin(fi)*cos(theta2)-cos(fi2)*sin(theta)*cos(fi)*cos(theta2)));
            % randomly invert or not invert the axis vector
            beta_rand=randi([0,1],1)*pi;
            % rotation matrix of the "x" unit vector, should be applied
            % before the rotation matrices used to rotate the "z" to be
            % parallel with the normal vector
            R_0=[cos(beta+beta_rand), -sin(beta+beta_rand), 0;...
                sin(beta+beta_rand), cos(beta+beta_rand), 0;...
                0, 0, 1];
            
            % rotating the linkers and connecting them to the contact points
            [labels, idx_mol]=m_gen_linker_rot(params, labels, link_vec_sign, R_0, R_y2, R_z2,  N_mol_cont(idx_cont), idx_mol, contact_point);
            
            labels.init_cont_coords(:,idx_cont)=contact_point;  % variable containing contact point coordinates in 3XN_cont array
        end
    end
end

end

