function [labels]=m_gen_mol_coord_vesicles(vesicle_coords, params)
% This function places the contact point on sphere surfaces (vesicles).
% First calculates the number of contact points on the individual
% vesicules, then places the contact point on them uniformly and connects
% the linkers to the contact points. The base of the half sphere of the
% linkers has to be tangential to the vesicle's surface.

% PARENT: m_gen_mol_coord.m
% CHILDS: -

% if fixed vesicle parameters were chosen
if isfield(params.sample,'ves_fix_N')
    N_ves=vesicle_coords.ves_N;  % number of vesicles
    N_cont_ves=params.sample.ves_fix_N;  % number of contact point on the individual vesicles
    R_ves=vesicle_coords.rad;    % radius of the individual vesicules
    
% if random vesicle parameters were chosen
elseif isfield(params.sample,'ves_Sig_ofmol')
    N_ves=vesicle_coords.ves_N;    % number of vesicles
    N_mean=params.sample.ves_N_ofmol;   % mean number of contact points on each vesicle
    N_var=params.sample.ves_Sig_ofmol;  % variance of the number of dye molecules on each vesicle
    P=1-N_var/N_mean;   % success probability in each trial
    
    % if the probability has negative or zero value...
    if P<=0
        P=0;        % count it as zero
        N_trials=0;
        disp('Too large variance was given for vesicules...')
    % if "N_mean" is zero
    elseif isnan(P)
        P=0;
        N_trials=0;
    % if P has meaningful value
    else
        N_trials=round(N_mean/P);    % number of trials in the binomial distribution
    end

    N_cont_ves=m_gen_random('binomial', N_trials, P, 1, N_ves);      % number of contact points on the individual vesicles 
    %N_cont_ves=binornd(N_trials, P, 1, N_ves);      % number of contact points on the individual vesicles 
    
    R_ves=vesicle_coords.rad;                    % radius of individual vesicles, following normal distribution
    
else
    error('Error in vesicles parameters');
end

N_cont=sum(N_cont_ves); % N of contact points in the sample

% getting linker vectors
[labels, N_mol_cont, link_vec_sign]=m_gen_linker(N_cont, params);

% indexing, helps determining which contact point to which vesicule belongs
ves_indices=zeros(1,N_cont);
idx_low=1;
idx_up=0;
for idx_ves=1:N_ves
    idx_up=idx_up+N_cont_ves(idx_ves);
    ves_indices(idx_low:idx_up)=idx_ves;
    idx_low=idx_up+1;
end

% boolean for whether the vesicules shall be spheres ("false") or
% ellipsoides ("true")
ellipsoidBool=params.sample.ves_ellipsoidBool;

if ~ellipsoidBool
    % spehere vesicules
    
    idx_mol=0;
    % calculate molecule positions
    for idx_cont=1:N_cont
        % get parameters of vesicle where the actual molecule is located
        idx_ves=ves_indices(idx_cont); % get index
        pos_ves=vesicle_coords.pos(:,idx_ves); % get center position
        rad_ves=R_ves(1, idx_ves); % get radius
        
        % generate spherical angles
        % for contact points, distributed uniformly, randomly on a sphere
        theta=acos(1-2*rand(1));
        fi=2*pi*rand(1);
        if isnan(fi)   % if the vedtor is parallel with the "z" axis...
            fi=0;      % let the azimuth agnle be zero
        end
        
        % rotation matrices, the base of the linker's half sphere
        % needs to be tangential to the vesicle's surface
        R_y=[cos(theta),0,sin(theta);...
            0,1,0;...
            -sin(theta),0,cos(theta)];
        R_z=[cos(fi),-sin(fi),0;...
            sin(fi),cos(fi),0;...
            0,0,1];
        
        % coordinates of contact point
        contact_point=pos_ves+rad_ves*[...
            sin(theta)*cos(fi);...
            sin(theta)*sin(fi);...
            cos(theta)];
        
        % rotating the linkers and connecting them to the contact points
        [labels, idx_mol]=m_gen_linker_rot(params, labels, link_vec_sign, R_y, R_z,  N_mol_cont(idx_cont), idx_mol, contact_point);
        
        labels.init_cont_coords(:,idx_cont)=contact_point;  % variable containing contact point coordinates in 3XN_cont array
    end
else
    % ellipsoidal vesicules
    
    
    % rotation angles of the ellipsoid
    theta_rot_ellipsoids=vesicle_coords.theta;      % angle for the ellipsoid's rotation around the "y" axis
    fi_rot_ellipsoids=vesicle_coords.fi;  % angle for the ellipsoid's rotation around the "z" axis
    
    idx_mol=0;
    for idx_cont=1:N_cont
        % get parameters of vesicle where the actual molecule is located
        idx_ves=ves_indices(idx_cont); %get index
        pos_ves=vesicle_coords.pos(:,idx_ves); %get center position
        theta_rot_ellipsoid_act=theta_rot_ellipsoids(idx_ves);
        fi_rot_ellipsoid_act=fi_rot_ellipsoids(idx_ves);
        
        % ellipsoid's axes
        first_semi_axis=R_ves(1, idx_ves);
        second_semi_axis=R_ves(2, idx_ves);
        third_semi_axis=R_ves(3, idx_ves);
        
        % Random spherical angles are generated to obtain homogenous
        % distribution on a sphere, then some of these angles are discarded in
        % such way that the resulting point have homogenous distribution on the
        % ellipsoid. The ellipsoid is parametrized by the spherical
        % coordinates and these are not the angles of the corresponding points
        % on the ellipsoid!!!
        discardBool=true;   % discard the generated angles, until it is "false"
        % generating the spherical angles until they are not discarded
        while discardBool
            % the spherical angles, generated in homogenous distribution on
            % a sphere
            theta=acos(1-2*rand(1));
            fi=2*pi*rand(1);
            if isnan(fi)   % if the vedtor is parallel with the "z" axis...
                fi=0;      % let the azimuth agnle be zero
            end
            
            if theta==0 || theta==pi
                % in this case, discard the generated angles and generate
                % new ones
                continue;
            end
            
            % unit element length on the ellipsoid belonging to the "theta"
            % coordinate
            theta_unit_element_vect=[...
                first_semi_axis*cos(theta)*cos(fi);...
                second_semi_axis*cos(theta)*sin(fi);...
                -third_semi_axis*sin(theta)];
            
            % unit element length on the ellipsoid belonging to the "fi"
            % coordinate
            fi_unit_element_vect=[...
                -first_semi_axis*sin(theta)*sin(fi);...
                second_semi_axis*sin(theta)*cos(fi);...
                0];
            
            % normal vector belonging to the generated point on the
            % ellipsoid, pointing outwards
            normal_vect=cross(theta_unit_element_vect,fi_unit_element_vect);
            % unit element size belonging to the generated point on the
            % ellipsoid in spherical coordinates
            unit_element_size=norm(normal_vect);
            
            % unit element size that would belong to a sphere with a radius of
            % "first_semi_axis" at the generated angles, in spherical
            % coordinates
            % the "unit_element_size" can not be higher than this
            spherical_unit_element_size=first_semi_axis*first_semi_axis*sin(theta);
            
            % random number that decides whether the generated point on the
            % ellipsoid should be discarded or not
            randForDiscarding=rand(1);
            
            % deciding whether the generated point on the ellipsoid should be
            % discarded or not
            if randForDiscarding<unit_element_size/spherical_unit_element_size
                % if he point should not be discarded
                
                % caese the while loop
                discardBool=false;
                
                % rotation matrices of the ellipsoid
                R_y_ellipsoid=[cos(theta_rot_ellipsoid_act),0,sin(theta_rot_ellipsoid_act);...
                    0,1,0;...
                    -sin(theta_rot_ellipsoid_act),0,cos(theta_rot_ellipsoid_act)];
                R_z_ellipsoid=[cos(fi_rot_ellipsoid_act),-sin(fi_rot_ellipsoid_act),0;...
                    sin(fi_rot_ellipsoid_act),cos(fi_rot_ellipsoid_act),0;...
                    0,0,1];
                
                % rotate the normal vector belonging to the generated point
                normal_vect_rot=R_z_ellipsoid*R_y_ellipsoid*normal_vect;
                
                % sperical angles of the rotated normal vector
                normal_vect_fi=atan2(normal_vect_rot(2), normal_vect_rot(1));
                normal_vect_theta=acos(normal_vect_rot(3)/unit_element_size);
                
                % rotation matrices, the base of the linker's half sphere
                % needs to be tangential to the vesicle's surface
                R_y_normal_vect=[cos(normal_vect_theta),0,sin(normal_vect_theta);...
                    0,1,0;...
                    -sin(normal_vect_theta),0,cos(normal_vect_theta)];
                R_z_normal_vect=[cos(normal_vect_fi),-sin(normal_vect_fi),0;...
                    sin(normal_vect_fi),cos(normal_vect_fi),0;...
                    0,0,1];
                
                % original Cartesian coordinates of the generated contact point
                contact_point_init=[...
                    first_semi_axis*sin(theta)*cos(fi);...
                    second_semi_axis*sin(theta)*sin(fi);...
                    third_semi_axis*cos(theta)];
                
                % apply the rotation of the ellipsoid to the contact point
                % coordinates
                contact_point_rot=R_z_ellipsoid*R_y_ellipsoid*contact_point_init;
                
                % translate the contact point with the central position of the
                % ellipsoid
                contact_point=pos_ves+contact_point_rot;
        
                % rotating the linkers and connecting them to the contact points
                [labels, idx_mol]=m_gen_linker_rot(params, labels, link_vec_sign, R_y_normal_vect, R_z_normal_vect,  N_mol_cont(idx_cont), idx_mol, contact_point);
                
                labels.init_cont_coords(:,idx_cont)=contact_point;  % variable containing contact point coordinates in 3XN_cont array
                
            else
                % if the point should be discarded
                discardBool=true;   % generate another angle for this contact point
            end
        end
    end
end

end