function [ mesh_coords ] = m_gen_read_stl( params )
% This function reads an *.stl file and and places the information in a
% cell containing two structures containing the normal vectors and the
% vertices of each triangle. Then rearranges the information into a more
% convenient format (which can be visualized in MATLAB). This format is
% structure containing three field. These fields are arrays containing the
% the coordinates of the individual vertices, normal vectors and the
% information about which vertice to which facet belongs.

% PARENT: m_gen_struct.m
% CHILDS: -

% VARIABLES

% filename: the name of the file to open
% scaling: used to increase the size of the pattern
% displacement: desired shift to the pattern
% line_act: content of the actual line in the file, string
% idx_facet: index of the actual triangle
% mesh_coords: structure containing tha face normals and the vertex coordinates
% temp: temporal structure that hellp arrange the information into more convenient format
% norm_vect: cell containing array containing the normal vector of the facet
% idx_vert: index of the vertex on the actual triangle
% vert_coords: cell containing 3x3 array containing the coordinates of vertices of the triangle
% N_facet: number of triangles
% indices: indices of the triangle's vertices in the "temp.vert_coords" array
% mesh_coords.norm_vect: array containing  the facet's normal vectors
% mesh_coords.vert_coords: array containing the coordinates of the individual vertices
% mesh_coords.facet_verts: array containing information about which vertice to which facet belongs


filename=params.sample.mesh_filename;       % the name of the file to open
scaling=params.sample.mesh_scaling;         % used to increase the size of the pattern
displacement=[params.sample.mesh_disp_x;params.sample.mesh_disp_y;params.sample.mesh_disp_z];  % desired shift to the pattern
theta=params.sample.mesh_rot_theta*pi/180;  % polar angle of rotation
fi=params.sample.mesh_rot_fi*pi/180;        % azimuth angle of rotation
% rotation matrices
R_y=[cos(theta),0,sin(theta);...
	0,1,0;...
    -sin(theta),0,cos(theta)];
R_z=[cos(fi),-sin(fi),0;...
    sin(fi),cos(fi),0;...
    0,0,1];

% if some of the displacement's values are empty
if size(displacement,1)~=3
   error('Wrong displacement for mesh'); 
end
fileID=fopen(filename);     % opens the given file
line_act=fgetl(fileID);     % reads the first line of the file

idx_facet=0;                % index of the actual triangle
mesh_coords=[];             % structure containing tha face normals and the vertex coordinates
temp=[];                    % temporal structure that hellp arrange the information into more convenient format
norm_vect=cell(1,1);               % cell array for facet normal vectors
vert_coords=cell(1,1);             % cell array for facet vertex coordinates

% reading the "*.stl" file

%  while it reaches the end of file
while ischar(line_act)
    % "facet normal ni nj nk" sings the first line of a facet
    if strncmp('facet',strip(line_act),5)
        idx_facet=idx_facet+1;   % index of the actual triangle
        % mesh_coords(idx_facet)=struct('norm_vect',zeros(3,1),'vert_coords',zeros(3,3));
        temp=str2double(strsplit(line_act));    % temporal variable for storing content of the line
        norm_vect{idx_facet,1}(1:3,1)=temp(4:6);  % normal vector of the facet
        idx_vert=0;         % index of the vertex on the actual triangle
        line_act=fgetl(fileID);     % read the next line
        % while it reaches the next facet or the end of file
        while ischar(line_act) && ~strncmp('facet',strip(line_act),5)
            % "vertex v1x v1y v1z" lines contains the vetex coordinates
            if strncmp('vertex',strip(line_act),6)
                idx_vert=idx_vert+1;
                temp=str2double(strsplit(line_act));
                vert_coords{idx_facet,1}(idx_vert,1:3)=R_z*R_y*((temp(3:5).*scaling)'+displacement);
            end
            line_act=fgetl(fileID);     % read the next line
        end
        % 3 vertices should belong to one facet
        if idx_vert~=3
        error('Error in mesh input file 1');
        end
    else
        line_act=fgetl(fileID);  % reads the lines until reaches the first curve 
    end
end

fclose(fileID);     % close the file

% if it did not read any coordinates...
if and(cellfun('isempty',norm_vect),cellfun('isempty',vert_coords))
    error('Error in mesh input file 2');
end

% arranging the infomation into more convenient format

N_facet=size(vert_coords,1);        % number of triangles

% temporal variables containing the arranged informations
% needs because "unique()" and "ismember()" does not have 'columns' property
temp=struct('norm_vect', [],'vert_coords', [],'facet_verts', []);
temp.norm_vect=cat(2,norm_vect{:,1});
temp.vert_coords=unique(cat(1,vert_coords{:,1}),'rows');
temp.facet_verts=zeros(N_facet,3);

% finding which vertice to which facet belongs
for idx_facet=1:N_facet
    for idx_vert=1:3
        indices=ismember(temp.vert_coords,vert_coords{idx_facet,1}(idx_vert,1:3),'rows');
        temp.facet_verts(idx_vert,idx_facet)=find(indices);
    end
end

% output variables containing the information arranged in convenient format
mesh_coords.norm_vect=temp.norm_vect;
mesh_coords.vert_coords=temp.vert_coords';
mesh_coords.facet_verts=temp.facet_verts;

end

