function [ vesicle_coords ] = m_gen_struct_vesicles( params )
% This function generates the center positions of vesicles. In case of
% user given coordinates it simply places the given coordinates in an
% array. Otherwise randomly places the vesicle centers in the given area.

% PARENT: m_gen_struct.m
% CHILDS: -

% if fixed coordinates were chosen
if isfield(params.sample,'ves_fix_N')

    % Could not find a more elegant solution
    error('For the synaptosome pattern, use the random vesicles pattern.')
    
% THE VESICLE COORDINATE GENERATION FOR THE SYNAPTOSOME PATTERN IS ADDED HERE: 
% if random coordinates were chosen
elseif isfield(params.sample,'ves_N_ofves')

    % these parameters must be set by hand: 
    synaptosome_posX=[1000,-2000,4000];     % "x" psoition of synaptosomes' centres
    synaptosome_posY=[500,-3000,2500];      % "y" psoition of synaptosomes' centres
    synaptosome_posZ=[0,200,-100];          % "z" psoition of synaptosomes' centres
    synaptosome_rad=repmat(300, size(synaptosome_posX));        % radius of each synaptosome
    synaptosome_vesN=repmat(params.sample.ves_N_ofves, size(synaptosome_posX));    % number of vesicles in each synaptosome

    synaptosome_pos=[synaptosome_posX; synaptosome_posY; synaptosome_posZ]; % 3x'synaptosome number' array containing the synaptosome positions
    synaptosome_N=size(synaptosome_pos,2);      % number of synaptosomes
    
    ves_N=sum(synaptosome_vesN);       % total number of vesicles in every synaptosome
    
    % initializing the structure array containing the synaptosome 
    % parameters
    synaptosome=repmat(struct('pos', [],'rad', [],'ves_N', [],'ves_pos', [],'ves_rad', []), [1,synaptosome_N]);
    
    % parameters describing the distribution of the vesicle radius
    R_mean=params.sample.ves_Rad_ofves(1);   % mean radius of vesicles
    R_sig=params.sample.ves_Sig_ofves;    % sigma of radius of vesicles
    
    % loop over the synaptosomes
    for idxSynap=1:synaptosome_N
        % parameters of the actual synaptosome
        synaptosome(idxSynap).pos=synaptosome_pos(:, idxSynap);     % position of the actual synaptosome
        synaptosome(idxSynap).rad=synaptosome_rad(1, idxSynap);     % radius of the actual synaptosome
        synaptosome(idxSynap).vesN=synaptosome_vesN(1, idxSynap);   % number of vesicles in the actual synaptosome
        synaptosome(idxSynap).ves_pos=[];       % position of the vesicles in the actual synaptosome
        synaptosome(idxSynap).ves_rad=[];       % radius of the vesicles in the actual synaptosome
        
        % loop throught the vesicles in the synaptosome
        idx_ves=0;      % index of the actual vesicle
        actVesTries_N=0;      % number of tries given in placing the actual vesicle (the vesicles can not overlap)
        while idx_ves<=synaptosome(idxSynap).vesN
            actVesTries_N=actVesTries_N+1;  % one more try was given in placing the actual vesicle
            
            % place the vesicle randomly in a cude determined by the
            % synaptosome radius, will discard it later, if it is outside
            % of the sphere
            actVesPos=rand(3,1)*synaptosome(idxSynap).rad+synaptosome(idxSynap).pos;
            
            % generate the radius for the actual vesicle
            actVesRad=R_mean+R_sig.*randn(1, 1);
            
            % examine whether the actual vesicle is inside the actual
            % synaptosome
            synaptosome_withinBool=sum((actVesPos-synaptosome(idxSynap).pos).^2)<=(synaptosome(idxSynap).rad-actVesRad)^2;
            
            % examine whether the actual vesicle overlaps with any of the
            % previously placed vesicles in the synaptosome
            if ~isempty(synaptosome(idxSynap).ves_pos)      % if it is not the first vesicle in the synaptosome
                vesDistances=sum((synaptosome(idxSynap).ves_pos-repmat(actVesPos, [1, size(synaptosome(idxSynap).ves_pos,2)])).^2,1);   % square of distances of the actual and the previously placed vesicles
                vesRadSums=synaptosome(idxSynap).ves_rad+repmat(actVesRad, [1, size(synaptosome(idxSynap).ves_rad,2)]);         % sum of radii of the actual and the previously placed vesicles
                vesDistances_RadSums_Bool=vesDistances>=vesRadSums.^2;      % examine whether actual vesicle overlaps with any other in the synaptosome
                vesicle_distBool=prod(vesDistances_RadSums_Bool);       % true if the actual vesicle does not overlap with any other in the synaptosome
            else        % if it is the first vesicle in the synaptosome
                vesicle_distBool=true;
            end
            
            % if the actual vesicle is inside the synaptosome and does not
            % overlap with any of the previously placed vesicles, add it to
            % the parameters of the actual synaptosome and leap to the next
            % vesicle
            if synaptosome_withinBool && vesicle_distBool
                idx_ves=idx_ves+1;
                actVesTries_N=0;
                synaptosome(idxSynap).ves_pos=[synaptosome(idxSynap).ves_pos, actVesPos];
                synaptosome(idxSynap).ves_rad=[synaptosome(idxSynap).ves_rad, actVesRad];
            end
            
            % if ttoo many tries was given in placing the vesicle, consider
            % not any more vesicle can be placed in this synatosome, leap
            % to the next synaptosome
            if actVesTries_N>100
                disp('Cannot place any more vesicle in this synaptosome.')
                break;
            end
            
        end
    end
    
    % concatenate the positions of the vesicles in different synaptosomes
    vesicle_coords.pos=[synaptosome(:).ves_pos];  % 3x(ves_fix_N/ves_N_ofves) array containing coordinates of vesicles
    
    % the total number of vesicels
    vesicle_coords.ves_N=ves_N;
    
    % concatenate the radii of the vesicles in different synaptosomes
    vesicle_coords.rad=[synaptosome(:).ves_rad];
    
    % rotation angles of the ellipsoid
    %theta_rot_ellipsoids=acos(1-2*rand(N_ves,1));     % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.theta=zeros(ves_N,1);      % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.fi=2*pi*rand(ves_N,1);  % angle for the ellipsoid's rotation around the "z" axis
    %fi_rot_ellipsoids=zeros(N_ves,1);  % angle for the ellipsoid's rotation around the "z" axis
    
else
    error('Error in vesicles coordinates');
end

end

