function [ F_aperture_func, spat_freq_x, spat_freq_y, im_coords_x, im_coords_y ] = m_gen_PSF_DFT( params, aperture_func )
% This function calculates the PSF in the image plane using 2D discrete
% Fourier-transform. Matrices required by the DFT are calculated only once
% for a given number of pixels taken into account (spatial frequencies are
% determined by the number of pixels taken into account).

% PARENT: m_gen_PSF_numerical_calc
% CHILD: -

global DFT_matrix_bool;

lambda=params.dye.lambda*1E-9;  % center wavelength of the emission in m
focal_plane_dist=params.aq.focal_plane_dist*1E-6;    % focal length under desing conditions in m

sampling_number_x=params.numcalc.sampling_number_x;
sampling_index_x=params.numcalc.sampling_index_x;
sampling_number_y=params.numcalc.sampling_number_y;
sampling_index_y=params.numcalc.sampling_index_y;
aperture_radius=params.numcalc.aperture_radius;
d_x_image=params.numcalc.d_x_image;
d_y_image=params.numcalc.d_y_image;
spat_freq_correction=params.numcalc.spat_freq_correction;

persistent d_x_ap ap_coords_x d_y_ap ap_coords_y    % coordiantes of the aperture's sampled points, needs to be calculated only once

if isempty(ap_coords_x)   % if the aperture's sampled points have yet to be calculated
    d_x_ap=2*aperture_radius/sampling_number_x;
    ap_coords_x=-aperture_radius+2*aperture_radius/sampling_number_x*(sampling_index_x-0.5);
end

if isempty(ap_coords_y)   % if the aperture's sampled points have yet to be calculated
    d_y_ap=2*aperture_radius/sampling_number_y;
    ap_coords_y=-aperture_radius+2*aperture_radius/sampling_number_y*(sampling_index_y-0.5);
end

persistent DFT_matrix_pers calculated_matrices
% DFT_matrix_pers: cell storing the calculated DFT matrices
% N_pixel_calculated: array storing whether the DFT matrix have been calculated for the given number of pixels

% if the calculation is executed with the other dye or with changed
% parameters (generate button has been pressed one more time)
if ~DFT_matrix_bool
    DFT_matrix_pers=[];
    calculated_matrices=[];
end

simpson_row_x=2*ones(1,sampling_number_x);
simpson_row_x(1,2:2:end)=4;
simpson_row_x(1,1)=1;
simpson_row_x(1,end)=1;
simpson_row_x=simpson_row_x/3;

simpson_row_y=2*ones(sampling_number_y,1);
simpson_row_y(2:2:end,1)=4;
simpson_row_y(1,1)=1;
simpson_row_y(end,1)=1;
simpson_row_y=simpson_row_y/3;
 
simpson_x=repmat(simpson_row_x,sampling_number_x,1);
simpson_y=repmat(simpson_row_y,1,sampling_number_y);

custom_row_x=ones(1,sampling_number_x);
custom_row_x(1,1)=9/24;
custom_row_x(1,2)=28/24;
custom_row_x(1,3)=23/24;
custom_row_x(1,end)=9/24;
custom_row_x(1,end-1)=28/24;
custom_row_x(1,end-2)=23/24;

custom_row_y=ones(sampling_number_y,1);
custom_row_y(1,1)=9/24;
custom_row_y(2,1)=28/24;
custom_row_y(3,1)=23/24;
custom_row_y(end,1)=9/24;
custom_row_y(end-1,1)=28/24;
custom_row_y(end-2,1)=23/24;

custom_x=repmat(custom_row_x,sampling_number_x,1);
custom_y=repmat(custom_row_y,1,sampling_number_y);

DFT_idx_x=find(calculated_matrices==d_x_image+1i*sampling_number_x);  % index of the calculated DFT matrix in "DFT_matrix_pers"

% constant for spatial frequency calculation
persistent spat_freq_const

if isempty(spat_freq_const)
    spat_freq_const=2*pi/focal_plane_dist/lambda*spat_freq_correction;
end

if isempty(DFT_idx_x) % if the DFT matrix have yet to be calculated
    im_coords_x=d_x_image*(sampling_number_x-1)*((sampling_index_x-1)/(sampling_number_x-1)-0.5);    % sampled point coordinates on the image plane
    % spatial frequencies in the image space coincide with the ones in the object space
    spat_freq_x=im_coords_x'*spat_freq_const;   % spatial frequencies for the Fourier-transform

    % the DFT martix have been calculated for a new number of pixels
    calculated_matrices=[calculated_matrices,d_x_image+1i*sampling_number_x];        % appending the new number of pixels for which the DFT matrix have benn calculated
    DFT_matrix_pers{end+1}=exp(-1i*spat_freq_x*ap_coords_x); % appending the new DFT matrix
    DFT_idx_x=find(calculated_matrices==d_x_image+1i*sampling_number_x);  % update index of the calculated DFT matrix in "DFT_matrix_pers"
end

DFT_idx_y=find(calculated_matrices==d_y_image+1i*sampling_number_y);  % index of the calculated DFT matrix in "DFT_matrix_pers"

if isempty(DFT_idx_y) % if the DFT matrix have yet to be calculated
    im_coords_y=d_y_image*(sampling_number_y-1)*((sampling_index_y-1)/(sampling_number_y-1)-0.5);    % sampled point coordinates on the image plane
    % spatial frequencies in the image space coincide with the ones in the object space
    spat_freq_y=im_coords_y'*spat_freq_const;   % spatial frequencies for the Fourier-transform

    % the DFT martix have been calculated for a new number of pixels
    calculated_matrices=[calculated_matrices,d_y_image+1i*sampling_number_y];        % appending the new number of pixels for which the DFT matrix have benn calculated
    DFT_matrix_pers{end+1}=exp(-1i*spat_freq_y*ap_coords_y); % appending the new DFT matrix
    DFT_idx_y=find(calculated_matrices==d_y_image+1i*sampling_number_y);  % update index of the calculated DFT matrix in "DFT_matrix_pers"
end

DFT_matrix_x=DFT_matrix_pers{DFT_idx_x};    % matrix for the actual DFT calculation
DFT_matrix_y=DFT_matrix_pers{DFT_idx_y};    % matrix for the actual DFT calculation
%DFT_matrix_x=DFT_matrix_pers{DFT_idx_x}.*custom_x;    % matrix for the actual DFT calculation
%DFT_matrix_y=DFT_matrix_pers{DFT_idx_y}.*custom_y;    % matrix for the actual DFT calculation
%DFT_matrix_x=DFT_matrix_pers{DFT_idx_x}.*simpson_x;    % matrix for the actual DFT calculation
%DFT_matrix_y=DFT_matrix_pers{DFT_idx_y}.*simpson_y;    % matrix for the actual DFT calculation


F_aperture_func=DFT_matrix_x*aperture_func*DFT_matrix_y*d_x_ap*d_y_ap*spat_freq_const/(2*pi);    % 2D DFT calculation
% "DFT_matrix" is symmetric, no need for transposing

% checking the equivalence of optical power transmitted through the aperture and hitting the detector
    %sum(sum(abs(aperture_func).^2))*d_x_ap*d_y_ap
    %sum(sum(abs(F_aperture_func).^2))*d_x_image*d_y_image
    
% checking the DFT matrix
    %figure()
    %mesh(abs(DFT_matrix_x))
    %figure()
    %mesh(real(DFT_matrix_x))
    %figure()
    %mesh(imag(DFT_matrix_x))

    %figure()
    %mesh(abs(DFT_matrix_y))
    %figure()
    %mesh(real(DFT_matrix_y))
    %figure()
    %mesh(imag(DFT_matrix_y))
    
    if ~exist('im_coords_y', 'var')
        im_coords_y=im_coords_x;
    end
    if ~exist('spat_freq_y', 'var')
        spat_freq_y=spat_freq_x;
    end
    
end

