%function [ PSF_image, idx_pix_x, idx_pix_y ] = m_gen_PSF_numerical_calc( params )
function [  ] = m_gen_PSF_numerical_calc(  )
% This function calculates the Frauhofer diffraction of point source. First
% determines the number of pixels affected by the PSF, then calcultes the
% aperture function and calculates the diffraction integral numeritically
% using pre-calculated matrices. Eventaully calcultes the intensity
% distribution of the PSF and pixelises it.

% PARENT: m_gen_PSF_calc
% CHILD: m_gen_PSF_aperture_func, m_gen_PSF_DFT, m_gen_PSF_pixelise

params.psf.type='scalar';
params.mol.pos=[0,0,0];
params.dye.lambda=600;
params.aq.frame_size=64;
params.aq.pixel_size=160;
params.aq.NA=1.4;
params.dye.em_N_ofphotonpersec=100000;
params.psf.colleff=0.4;
params.aq.focal_plane_dist=300;
params.psf.sample_depth=5;
params.aq.n_immersion=1.518;        % refractive index of immersion medium
params.psf.n_sample=1.331;          % refractive index of water at 650 nm
params.psf.cone_angle=2*pi/180;
params.psf.el_field_dir=[0,0,1];
params.psf.tau_rot=1000;
params.psf.tau_fluor=1;
params.psf.anisotropy_bool=false;
params.meas.polarization_bool=false;
global DFT_matrix_bool
global aperture_func_bool
global Zernike_const_aberration_bool
DFT_matrix_bool=false;
aperture_func_bool=false;
Zernike_const_aberration_bool=false;

% orientation of the dipole
params.mol.init_orientation(1)=0*pi/180;       % fi
params.mol.init_orientation(2)=0*pi/180;       % theta

psf_type=params.psf.type;

sampling_number_x=67;        % number of points (sampled points) in "x" dimensions in which the values of the aperture function and the PSF shall be calculated
sampling_index_x=1:sampling_number_x;   % indices used for the "x" sampled points
sampling_number_y=67;        % number of points (sampled points) in "y" dimensions in which the values of the aperture function and the PSF shall be calculated
sampling_index_y=1:sampling_number_y;   % indices used for the "y" sampled points

% positions of the molecule in the focal plane
x_pos=params.mol.pos(1)*1E-9;
y_pos=params.mol.pos(2)*1E-9;
% positions of the molecule on the optical axis
z_pos=params.mol.pos(3)*1E-9;        % "z" position of the molecule under design conditions in m

lambda=params.dye.lambda*1E-9;               % wavelength of emitted light in m

N_pixel_x=params.aq.frame_size;         % number of pixels on the frames in one dimension
N_pixel_y=params.aq.frame_size;         % number of pixels on the frames in one dimension
pixel_size=params.aq.pixel_size*1E-9;   % size of pixels in m
NA=params.aq.NA;                        % objective's numerical aperture
focal_plane_dist=params.aq.focal_plane_dist*1E-6;     % distante of the object from the objective's aperture under desing conditions in m
n_i=params.aq.n_immersion;                          % refractive index of immersion medium

polarization_bool=params.meas.polarization_bool;
anisotropy_bool=params.psf.anisotropy_bool;
if anisotropy_bool
    tau_r=params.psf.an_tau_rot;
    tau_f=params.psf.an_tau_fluor;
end

% approximating the PSF size in case of aberration free imaging
% is this for water immersion objective only ???
sigma_0=0.25*lambda/NA;       % standard deviation of the Gaussian PSF fitted on the diffraction limited PSF
w_0=2*sigma_0;                  % width of the field's amplitude of the gaussian beam
% taken from "Accuracy of the Gaussian Point Spread Function model in 2D localization microscopy"

% calculating the beam width assuming Gaussian beam
z0=pi*w_0^2/lambda;
w=w_0*sqrt(1+(z_pos/z0)^2);

% beam radii in "x" and "y" directions
w_x=w;
w_y=w;

% range of affected area in "x" and "y" directions
psf_range_x=2*w_x;
psf_range_y=2*w_y;
%psf_range_x=750*1E-9;
%psf_range_y=750*1E-9;
[ d_x_image, psf_pos_rem_x, idx_pix_x, idx_psf_sampling_x ] = m_gen_PSF_image_sampling_points( x_pos, psf_range_x, N_pixel_x, pixel_size, sampling_number_x );
[ d_y_image, psf_pos_rem_y, idx_pix_y, idx_psf_sampling_y ] = m_gen_PSF_image_sampling_points( y_pos, psf_range_y, N_pixel_y, pixel_size, sampling_number_y );

% if the molecule's PSf is out of the active area
if isempty(idx_pix_x) || isempty(idx_pix_y)
   switch psf_type
       case 'scalar'
           PSF_image{1}=zeros(length(idx_pix_x),length(idx_pix_y));
       case 'vectorial'
           PSF_image{1}=zeros(length(idx_pix_x),length(idx_pix_y));
           PSF_image{2}=zeros(length(idx_pix_x),length(idx_pix_y));
   end
   return   % stop execution and return to the parent function
end

% calculating the objective's aperture's radius from the NA, focal length
% and the refractive index of immersion medium
% the refractive index of the immersion medium and the cover slip are
% assumed to be equal
alpha_i=asin(NA/n_i);   %the maximal half-angle of the cone of light in the immersion medium

% if the angles are complex, e.g. wrong values were given either for the
% refractive indices or for the numerical aperture
if ~isreal(alpha_i)
    error('Wrong immersion medium RI for the given numerical aperture.')
end

aperture_radius=focal_plane_dist*tan(alpha_i);  % radius of the objective's aperture in m

% collecting parameters for the child functions
params.numcalc.sampling_number_x=sampling_number_x;
params.numcalc.sampling_index_x=sampling_index_x;
params.numcalc.sampling_number_y=sampling_number_y;
params.numcalc.sampling_index_y=sampling_index_y;
params.numcalc.aperture_radius=aperture_radius;
params.numcalc.psf_pos_rem_x=psf_pos_rem_x;
params.numcalc.psf_pos_rem_y=psf_pos_rem_y;
params.numcalc.d_x_image=d_x_image;
params.numcalc.d_y_image=d_y_image;
params.numcalc.spat_freq_correction=n_i*focal_plane_dist*NA/aperture_radius/n_i;   % correction for spatial frequencies in order to be able to calculate the optical imaging with the objective's aperture radius and its distance from the object

[ aperture_func ]=m_gen_PSF_aperture_func(params);      % calculating the electric field in the aperture

c=299792458;            % speed of light in m/s
eps_0=8.854187817620*10^-12;    % vacuum permittivity in SI units
k=2*pi/lambda;                  % wave number in "1/m"
h_mod=6.626070040*1E-34/(2*pi); % modified Planck's contant in SI units


[E, spat_freq_x, spat_freq_y, im_coords_x, im_coods_y]=m_gen_PSF_DFT(params, aperture_func{1});
PSF{1}=abs(E).^2*c*eps_0/2*d_x_image*d_y_image/(h_mod*c*k); % optical power belonging to individual sampled points

%width_y=aperture_radius/10;
%width_x=aperture_radius/10;
%E2=exp(-spat_freq_y.^2*(width_y^4/4))*exp(-spat_freq_x'.^2*(width_x^4/4));
ro_2_pi=sqrt(repmat(spat_freq_y,1, sampling_number_y).^2+repmat(spat_freq_x', sampling_number_x,1).^2);

E2=besselj(1,ro_2_pi*aperture_radius)./(ro_2_pi*aperture_radius)/(3/2*pi);
E2(isnan(E2))=0.5/(3/2*pi);

%figure
%mesh(abs(aperture_func{1}))
%figure
%mesh(abs(E))
%figure
%mesh(abs(E2))

figure
mesh(abs(E2./E))

[PSF_image]=m_gen_PSF_pixelise( params, PSF, idx_pix_x, idx_pix_y, idx_psf_sampling_x ,idx_psf_sampling_y);          % pixelated PSF


% the PSF using Gaissian model, used checking the numerically calculated PSF
%sigma_x=w_x/2;
%sigma_y=w_y/2;
%x_gauss_sampled_int=(sampling_index_x-1-(sampling_number_x-1)/2)*d_x_image;
%x_gauss_sampled=repmat(x_gauss_sampled_int',1,length(x_gauss_sampled_int));
%y_gauss_sampled_int=(sampling_index_y-1-(sampling_number_y-1)/2)*d_y_image;
%y_gauss_sampled=repmat(y_gauss_sampled_int,length(y_gauss_sampled_int),1);
%PSF_Gauss{1}=exp(-(x_gauss_sampled).^2/2/sigma_x^2).*exp(-(y_gauss_sampled).^2/2/sigma_y^2);
%figure()
%mesh(PSF_Gauss{1})
%[PSF_image]=m_gen_PSF_pixelise( params, PSF_Gauss, idx_pix_x, idx_pix_y, idx_psf_sampling_x ,idx_psf_sampling_y);

% testing the intensity PSFs
    %figure
    %mesh(PSF{1})
    %figure
    %mesh(PSF{2})
    
    %figure
    %mesh(PSF{1}(idx_psf_sampling_x{2},idx_psf_sampling_y{2}))
    %figure
    %mesh(PSF{1}(idx_psf_sampling_x{2},idx_psf_sampling_y{4}))
    %figure
    %mesh(PSF{1}(idx_psf_sampling_x{3},idx_psf_sampling_y{3}))
    
% testing the electric fields on the aperture
    %figure()
    %mesh(abs(aperture_func{1}))
    %figure()
    %mesh(imag(aperture_func{1}))
    %figure()
    %mesh(imag(aperture_func{1}))
    %figure()
    %mesh(abs(aperture_func{2}))
    %figure()
    %mesh(imag(aperture_func{2}))
    %figure()
    %mesh(imag(aperture_func{2}))

% testing the pixelizes PSF
    %figure()
    %mesh(PSF_image{1})

    %PSF_image{1}

% the number of photons hitting the detector
    %sum(sum(PSF_image{1}))
    %sum(sum(PSF_image{2}))

    %idx_psf_sampling_x{:}
    %idx_psf_sampling_y{:}
    %psf_area_center
    
end